@@ -1,11 +1,11 @@
 /*
  * AstroPanel.m
  *
- * Copyright 1999-2004 by vhf interservice GmbH
+ * Copyright 1999-2005 by vhf interservice GmbH
  * Author:   Georg Fleischmann
  *
  * Created:  1995-08-10
- * Modified: 2004-06-04
+ * Modified: 2005-09-13
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the vhf Public License as
@@ -31,6 +31,9 @@
 #include "AstroPanel.h"
 #include "../astroMessages.h"
 #include "APBasicLevel.h"
+#if ! USE_TABVIEW	// OpenStep
+#    include "../Cenon/UITabActionCell.subproj/UITabActionCell.h"
+#endif
 
 @implementation AstroPanel
 
@@ -38,40 +41,48 @@
 - (BOOL)canBecomeKeyWindow	{ return YES; }
 
 - init
-{   NSTabViewItem	*item;
+{   NSTabViewItem		*item;
+    NSNotificationCenter	*notificationCenter = [NSNotificationCenter defaultCenter];
 
     [super init];
-
     [tabView setDelegate:self];
+    levelCnt = 0;
+
+    /* add observer to add tabs */
+    [notificationCenter addObserver:self
+                           selector:@selector(addTabNotification:)
+                               name:AstroPanelAddTabNotification
+                             object:nil];
 
     item = [[NSTabViewItem alloc] initWithIdentifier:@"Chart"];
     [item setLabel:@"Chart"];
     [item setView:[[self windowAt:AP_CHART] contentView]];
     [tabView addTabViewItem:item];
-    activeWindow = chartWindow;
+    windows[levelCnt] = activeWindow = chartWindow;
+    levelCnt++;
 
     item = [[NSTabViewItem alloc] initWithIdentifier:@"Map"];
     [item setLabel:TABMAP_STRING];
     [item setView:[[self windowAt:AP_MAP] contentView]];
     [tabView addTabViewItem:item];
+    windows[levelCnt] = mapWindow;
+    levelCnt++;
 
     item = [[NSTabViewItem alloc] initWithIdentifier:@"Search"];
     [item setLabel:TABSEARCH_STRING];
     [item setView:[[self windowAt:AP_SEARCH] contentView]];
     [tabView addTabViewItem:item];
-
-    /*item = [[NSTabViewItem alloc] initWithIdentifier:@"Mandala"];
-    [item setLabel:@"Mandala"];
-    [item setView:[[self windowAt:AP_MANDALA] contentView]];
-    [tabView addTabViewItem:item];*/
+    windows[levelCnt] = searchWindow;
+    levelCnt++;
 
     return self;
 }
 
 /* delegate: update view of tabViewItem
  */
-- (void)tabView:(NSTabView*)tabView willSelectTabViewItem:(NSTabViewItem*)tabViewItem
-{
+- (void)tabView:(NSTabView*)tabV willSelectTabViewItem:(NSTabViewItem*)tabViewItem
+{   int	level = [tabV indexOfTabViewItem:tabViewItem];
+
     /* click on tab -> set parent view
      */
     if ( !setSubLevel )
@@ -94,11 +105,11 @@
             [tabViewItem setView:[[self windowAt:AP_SEARCH] contentView]];
             activeWindow = searchWindow;
         }
-        else if ( [[tabViewItem identifier] isEqual:@"Mandala"] && activeWindow != mandalaWindow )
+        else if (level < levelCnt)
         {
             [activeWindow displayWillEnd];
-            [tabViewItem setView:[[self windowAt:AP_MANDALA] contentView]];
-            activeWindow = mandalaWindow;
+            [tabViewItem setView:[[self windowAt:level] contentView]];
+            activeWindow = windows[level];
         }
         [activeWindow update:self];
     }
@@ -112,21 +123,17 @@
 
     [activeWindow displayWillEnd];
     setSubLevel = YES;
-    switch ( level )
+    if ( level < levelCnt )
     {
-     	case AP_CHART:
-     	case AP_MAP:
-     	case AP_SEARCH:
-     	case AP_MANDALA:
-            item = [tabView tabViewItemAtIndex:level];
-            if ( [item view] != newView )
-            {   [[item view] retain];
+        item = [tabView tabViewItemAtIndex:level];
+        if ( [item view] != newView )
+        {   [[item view] retain];
 #ifdef GNUSTEP_BASE_VERSION
-                [tabView selectTabViewItem:nil];	// GNUstep: old views are not removed
+            [tabView selectTabViewItem:nil];	// GNUstep: old views are not removed
 #endif
-                [item setView:newView];
-                [tabView selectTabViewItemAtIndex:level];
-	    }
+            [item setView:newView];
+            [tabView selectTabViewItemAtIndex:level];
+            }
     }
     setSubLevel = NO;
     activeWindow = [self windowAt:level];
@@ -136,8 +143,17 @@
 #else	// OpenStep
 
 - init
-{
+{   NSNotificationCenter	*notificationCenter = [NSNotificationCenter defaultCenter];
+
+    /* add observer to add tabs */
+    [notificationCenter addObserver:self
+                           selector:@selector(addTabNotification:)
+                               name:AstroPanelAddTabNotification
+                             object:nil];
+
+    levelCnt = [levelRadio numberOfColumns];
     [self clickLevel:self];
+
     return self;
 }
 
@@ -151,34 +167,20 @@
 - (void)setLevelAt:(int)level
 {
     [activeWindow displayWillEnd];
-    if (level < 10)
+    if (level < levelCnt)
         [levelRadio selectCellAtRow:0 column:level];
-    switch ( level )
+
+    if (level < levelCnt )
     {
-        case AP_CHART:		// astro chart
-            [self windowAt:AP_CHART];
-            [self setLevelView:[chartWindow contentView]];
-            activeWindow = chartWindow;
-            break;
-        case AP_MAP:		// map
-            [self windowAt:AP_MAP];
-            [self setLevelView:[mapWindow contentView]];
-            activeWindow = mapWindow;
-            break;
-        case AP_SEARCH:		// search
-            [self windowAt:AP_SEARCH];
-            [self setLevelView:[searchWindow contentView]];
-            activeWindow = searchWindow;
-            break;
-        case AP_MANDALA:	// mandala
-            [self windowAt:AP_MANDALA];
-            [self setLevelView:[mandalaWindow contentView]];
-            activeWindow = mandalaWindow;
-            break;
-        default:
-            [self setLevelView:NULL];
-            activeWindow = self;
-            return;
+        [self windowAt:level];
+        [self setLevelView:[windows[level] contentView]];
+        activeWindow = windows[level];
+    }
+    else
+    {
+        [self setLevelView:NULL];
+        activeWindow = self;
+        return;
     }
 
     [activeWindow update:self];
@@ -197,11 +199,10 @@
 #endif	// end OpenStep
 
 - (void)update:sender
-{
-    [chartWindow update:sender];
-    [mapWindow update:sender];
-    [searchWindow update:sender];
-    [mandalaWindow update:sender];
+{   int	i;
+
+    for (i=0; i < levelCnt; i++)
+        [windows[i] update:sender];
 }
 
 - (id)activeWindow
@@ -219,6 +220,7 @@
                     NSLog(@"Cannot load APChart model");
                 [[chartWindow init] setWindow:self];
                 [[chartWindow contentView] retain];	// GNUstep
+                windows[AP_CHART] = chartWindow;
             }
             return chartWindow;
         case AP_MAP:
@@ -227,6 +229,7 @@
                     NSLog(@"Cannot load APMap model");
                 [[mapWindow init] setWindow:self];
                 [[mapWindow contentView] retain];	// GNUstep
+                windows[AP_MAP] = mapWindow;
             }
             return mapWindow;
         case AP_SEARCH:
@@ -235,17 +238,11 @@
                     NSLog(@"Cannot load APSearch model");
                 [[searchWindow init] setWindow:self];
                 [[searchWindow contentView] retain];	// GNUstep
+                windows[AP_SEARCH] = searchWindow;
             }
             return searchWindow;
-        case AP_MANDALA:
-            if ( !mandalaWindow )
-            {	if (![NSBundle loadModelNamed:@"APMandala" owner:self])
-                    NSLog(@"Cannot load APMandala model");
-                [[mandalaWindow init] setWindow:self];
-                [[mandalaWindow contentView] retain];	// GNUstep
-            }
-            return mandalaWindow;
-        default: return nil;
+        default:
+            return (level < levelCnt) ? windows[level] : nil;
     }
 }
 
@@ -286,4 +283,41 @@
     }
 }
 
+
+/*
+ * Notifications
+ */
+
+- (void)addTabNotification:(NSNotification*)notification
+{   APBasicLevel	*panel = [notification object];
+    int			level = levelCnt;
+
+    windows[level] = [panel retain];
+    levelCnt ++;
+    [panel setWindow:self];
+    [[panel contentView] retain];	// GNUstep
+
+#if USE_TABVIEW	// GNUstep, MAC-OS X
+    {   NSTabViewItem	*item;
+
+        item = [[NSTabViewItem alloc] initWithIdentifier:[panel name]];
+        [item setLabel:[panel name]];
+        [item setView:[panel contentView]];
+        [tabView addTabViewItem:item];
+    }
+#else
+    {   UITabActionCell	*item;
+        NSSize		size;
+
+        [levelRadio addColumn];
+        item = [levelRadio cellAtRow:0 column:level];
+        [item setStringValue:[panel name]];
+        size = [levelRadio bounds].size;
+        [levelRadio setCellSize:NSMakeSize(size.width/levelCnt, size.height)];
+        [self display];
+        [self flushWindow]; 
+    }
+#endif
+}
+
 @end
